# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""


import matplotlib
import matplotlib.transforms
import wx

import copy
import pylab
import numpy
import pyphant.core.Connectors
from pyphant.core import DataContainer
from pyphant.wxgui2.DataVisReg import DataVisReg
from pyphant.quantities import isQuantity, Quantity
from pyphant.quantities.ParseQuantities import str2unit

class PlotPanel (wx.PyPanel):
    """The PlotPanel has a Figure and a Canvas. OnSize events simply set a
    flag, and the actual resizing of the figure is triggered by an Idle event."""
    def __init__( self, parent, color=None, dpi=None, **kwargs ):
        from matplotlib.backends.backend_wxagg import (FigureCanvasWxAgg,
                                                       NavigationToolbar2WxAgg)
        from matplotlib.figure import Figure

        # initialize Panel
        if 'id' not in kwargs.keys():
            kwargs['id'] = wx.ID_ANY
        if 'style' not in kwargs.keys():
            kwargs['style'] = wx.NO_FULL_REPAINT_ON_RESIZE
        wx.Panel.__init__( self, parent, **kwargs )
        self.parent = parent

        # initialize matplotlib stuff
        self.figure = Figure( None, dpi )
        self.canvas = FigureCanvasWxAgg( self, -1, self.figure )
        self.ax = self.figure.add_subplot(111)
        self.figure.subplots_adjust(hspace=0.8)
        self.toolbar = NavigationToolbar2WxAgg(self.canvas)

        vSizer = wx.BoxSizer(wx.VERTICAL)
        vSizer.Add(self.canvas, 1, wx.LEFT | wx.TOP | wx.GROW)
        vSizer.Add(self.toolbar, 0, wx.EXPAND)
        self.draw()
        self.SetAutoLayout(True)
        self.SetSizer(vSizer)
        self.Layout()

    def draw(self):
        pass # abstract, to be overridden by child classes


class F(pylab.Formatter):
    def __init__(self, container, *args, **kwargs):
        self.container=container
    def __call__(self, x, pos=None):
        try:
            return str(x*self.container.unit)
        except IndexError, error:
            return str(x)


class OscPlotPanel(PlotPanel):
    x_key=0
    y_key=1
    c_key=2
    radius=Quantity('0.4mm')
    vmin = None
    vmax = None
    def __init__(self, parent, dataContainer):
        self.dataContainer = dataContainer
        PlotPanel.__init__(self, parent)

    def draw(self):
        self.x = self.dataContainer[self.x_key]
        self.y = self.dataContainer[self.y_key]
        self.c = self.dataContainer[self.c_key]
        if self.vmin is not None:
            self.vmin /= self.c.unit
        if self.vmax is not None:
            self.vmax /= self.c.unit
        self.figure.clear()
        self.ax = self.figure.add_subplot(111)
        self.figure.subplots_adjust(hspace=0.8)
        aspect = self.y.unit/self.x.unit
        if not isinstance(aspect, Quantity):
            self.ax.set_aspect(aspect)
        try:
            if self.c.mask!=None:
                mask = numpy.logical_not(self.c.mask)
                x = self.x.data[mask]
                y = self.y.data[mask]
                c = self.c.data[mask]
            else:
                x = self.x.data
                y = self.y.data
                c = self.c.data
            self.scat = self.ax.scatter(x,y,
                                        s=numpy.pi*(self.radius/self.x.unit)**2,
                                        c=c,
                                        vmin=self.vmin, vmax=self.vmax)
            self.colorbar = self.figure.colorbar(self.scat, format=F(self.c),
                                                 ax=self.ax)
            self.rescale(self.ax)
        except Exception, e:
            print e
        self.ax.set_xlabel(self.x.label)
        self.ax.set_ylabel(self.y.label)
        try:
            self.ax.set_title(self.dataContainer.attributes['title'])
        except KeyError,TypeError:
            pass
        self.ax.callbacks.connect('xlim_changed', self.rescale)
        self.ax.callbacks.connect('ylim_changed', self.rescale)
        self.canvas.draw()

    def rescale(self, ax):
        scale = self.ax.transData.get_affine().get_matrix().copy()
        scale[0,2]=0
        scale[1,2]=0
        scale[1,1]=scale[0,0]
        self.scat.set_transform(matplotlib.transforms.Affine2D(scale))


class ConfigurationPanel(wx.PyPanel):
    def __init__( self, parent, dataContainer, plot_panel):
        wx.Panel.__init__( self, parent)
        self.dataContainer = dataContainer
        self.plot_panel = plot_panel
        self.columns = [name for name in self.dataContainer.longnames.keys()
                        if isinstance(self.dataContainer[name],
                                      DataContainer.FieldContainer)]
        sizer = wx.FlexGridSizer(2, 6)
        sizer.Add(wx.StaticText(self, label="x axis: "))
        sizer.Add(wx.StaticText(self, label="y axis: "))
        sizer.Add(wx.StaticText(self, label="Color variable: "))
        sizer.Add(wx.StaticText(self, label="Minimal value: "))
        sizer.Add(wx.StaticText(self, label="Maximal value: "))
        sizer.Add(wx.StaticText(self, label="Radius: "))
        self.independentCombo = wx.ComboBox(self,
                                            style=wx.CB_READONLY | wx.CB_SORT,
                                            choices=self.columns)
        self.independentCombo.SetStringSelection(self.columns[0])
        longest = ''
        for s in self.columns:
            if len(s)>len(longest):
                longest = s
        offset = self.independentCombo.Size \
            - self.independentCombo.GetTextExtent(longest)
        maxSize = wx.Size(*self.independentCombo.GetTextExtent("White Reference Count somemore"))
        maxSize += offset
        self.independentCombo.MinSize = maxSize
        sizer.Add(self.independentCombo)
        self.dependentCombo = wx.ComboBox(self,
                                          style=wx.CB_READONLY | wx.CB_SORT,
                                          choices=self.columns)
        self.dependentCombo.SetStringSelection(self.columns[1])
        self.dependentCombo.MinSize = maxSize
        sizer.Add(self.dependentCombo)
        self.colorCombo = wx.ComboBox(self,
                                      style=wx.CB_READONLY | wx.CB_SORT,
                                      choices=self.columns)
        self.colorCombo.SetStringSelection(self.columns[2])
        self.colorCombo.MinSize = maxSize
        sizer.Add(self.colorCombo)
        self.vsize = wx.Size(*self.colorCombo.GetTextExtent('1234567890 nm')) + offset
        self.vmin_text = wx.TextCtrl(self, size=self.vsize,
                                     style=wx.TE_PROCESS_ENTER)
        sizer.Add(self.vmin_text)
        self.vmax_text = wx.TextCtrl(self, size=self.vsize,
                                     style=wx.TE_PROCESS_ENTER)
        sizer.Add(self.vmax_text)
        self.radius_text = wx.TextCtrl(self, size=self.vsize,
                                       style=wx.TE_PROCESS_ENTER)
        self.radius_text.Value = '0.4 mm'
        sizer.Add(self.radius_text)
        self.SetSizer(sizer)
        self.Centre()
        self.Bind(wx.EVT_COMBOBOX, self.changeColumns, self.independentCombo)
        self.Bind(wx.EVT_COMBOBOX, self.changeColumns, self.dependentCombo)
        self.Bind(wx.EVT_COMBOBOX, self.changeColumns, self.colorCombo)
        self.Bind(wx.EVT_TEXT_ENTER, self.changeColumns, self.vmin_text)
        self.Bind(wx.EVT_TEXT_ENTER, self.changeColumns, self.vmax_text)
        self.Bind(wx.EVT_TEXT_ENTER, self.changeColumns, self.radius_text)

    def changeColumns(self, event=None):
        independentVariable = self.independentCombo.GetValue()
        dependentVariable = self.dependentCombo.GetValue()
        colorVariable = self.colorCombo.GetValue()
        if (u''==independentVariable
            or u''==dependentVariable
            or u''==colorVariable):
            return
        self.plot_panel.x_key = independentVariable
        self.plot_panel.y_key = dependentVariable
        if self.plot_panel.c_key != colorVariable:
            self.plot_panel.c_key = colorVariable
            field = self.dataContainer[colorVariable]
            vmin = numpy.nanmin(field.data).round()*field.unit
            self.vmin_text.Value=str(vmin)
            vmax = numpy.nanmax(field.data).round()*field.unit
            self.vmax_text.Value=str(vmax)
        self.plot_panel.vmin = str2unit(self.vmin_text.Value)
        self.plot_panel.vmax = str2unit(self.vmax_text.Value)
        self.plot_panel.radius = str2unit(self.radius_text.Value)
        self.plot_panel.draw()
        self.GetSizer().Fit(self)

class PlotFrame(wx.Frame):
    name='Plot frame'
    def __init__(self, dataContainer, show=True):
        wx.Frame.__init__(self, None, -1, "Visualize "+dataContainer.longname)
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.plot_panel = OscPlotPanel(self, dataContainer)
        self.configuration_panel = ConfigurationPanel(self, dataContainer,
                                                      self.plot_panel)
        self.sizer.Add(self.configuration_panel, 0, wx.EXPAND)
        self.sizer.Add(self.plot_panel, 1, wx.LEFT | wx.TOP | wx.GROW)
        self.SetSizer(self.sizer)
        self.Centre()
        self.Show(True)

    def createVisualizerList(self):
        ids={}
        if isinstance(self.dataContainer, DataContainer.SampleContainer):
            data_type = pyphant.core.Connectors.TYPE_ARRAY
        elif isinstance(self.dataContainer, DataContainer.FieldContainer):
            data_type = pyphant.core.Connectors.TYPE_IMAGE
        for visualizer in DataVisReg.getInstance().getVisualizers(data_type):
            id=wx.NewId()
            ids[id]=visualizer
            #self.GetCanvas().Bind(wx.EVT_MENU, self.visualize, id=id)
        return ids

    def OnExit(self, event):
        self.Close()




DataVisReg.getInstance().registerVisualizer(pyphant.core.Connectors.TYPE_ARRAY, PlotFrame)



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
This module provides the WebInterface class for the KnowledgeNode
as well as some HTTP and HTML helper classes.
"""

from __future__ import with_statement
from types import (ListType, TupleType)
from pyphant.core.bottle import (template, send_file, request)
from pyphant.core.Helpers import getPyphantPath
from urllib import urlencode
from pyphant.core.KnowledgeNode import RemoteError
from pyphant.core.KnowledgeManager import DCNotFoundError
from pyphant.core.SQLiteWrapper import SQLiteWrapper
import os
import pkg_resources


def cond(condition, results):
    if condition:
        return results[0]
    else:
        return results[1]


def validate(value_validator_message_list):
    valid = True
    error_msg = ""
    for value, validator, message in value_validator_message_list:
        if not validator(value):
            valid = False
            if "%s" in message:
                message = message % value.__repr__()
            error_msg += "<p>%s</p>\n" % message
    if not valid:
        raise ValueError(error_msg)


def validate_keys(keys, mapping):
    try:
        validate([(k, lambda k: k in mapping, "Missing parameter: %s") \
                  for k in keys])
    except ValueError, verr:
        return template('message', heading='Parameter Error',
                        message=verr.args[0], back_url='/')
    return True


def order_bar(keys, order_by, order_asc):
    keydict = dict([(key, key) for key in keys])
    extra = cond(order_asc, (" +", " -"))
    keydict[order_by] = "<i>%s</i>%s" % (order_by, extra)
    return [HTMLJSButton(keydict[key],
                         "set_order_by('%s');" % key) for key in keys]


def shorten(tolong, premax=7, postmax=15):
    if len(tolong) > premax + postmax + 3:
        if postmax == 0:
            post = ""
        else:
            post = tolong[-postmax:]
        return tolong[:premax] + "..." + post
    else:
        return tolong


def html_latex(mathstr):
    return '<pre class="LaTeX">$%s$</pre>' % mathstr


def nice(value, key, do_shorten):
    if isinstance(value, (TupleType, ListType)):
        assert isinstance(key, (TupleType, ListType))
        nicelist = [nice(singleval, singlekey, do_shorten) \
                    for singleval, singlekey in zip(value, key)]
        if isinstance(value, TupleType):
            return tuple(nicelist)
        else:
            return nicelist
    else:
        if do_shorten and key in ['longname', 'creator', 'machine']:
            return shorten(value)
        elif key == 'shortname':
            return html_latex(value)
        else:
            return value


class HTMLLink():
    """
    This class provides HTML code for hyperlinks.
    """

    def __init__(self, url, linkobj, target=None):
        """
        url -- self explanatory
        linkobj -- obj the user sees to click on
        target -- link target, e.g. '_blank'
        """
        self._url = url
        self._linkobj = linkobj
        self._target = target

    def getHTML(self):
        """
        Returns the hyperlink as HTML code.
        """
        targetstring = ""
        if self._target != None:
            targetstring = ' target="%s"' % (self._target, )
        if hasattr(self._linkobj, 'getHTML'):
            linkhtml = self._linkobj.getHTML()
        else:
            linkhtml = str(self._linkobj)
        return '<a href="%s"%s>%s</a>'\
               % (self._url, targetstring, linkhtml)


class HTMLDropdown(object):
    def __init__(self, name, options, select=None, onchange=None):
        self.name = name
        self.options = options
        self.select = select
        self.onchange = onchange

    def getHTML(self):
        htmlopt = '    <option value="%s"%s>%s</option>\n'
        onchg = cond(self.onchange is None,
                     ('', 'onchange="%s"' % self.onchange))
        html = '<select name="%s" size="1" %s>\n' % (self.name, onchg)
        for item in self.options:
            try:
                value, label = item
            except ValueError:
                value = item
                label = item
            selected = ""
            if value == self.select:
                selected = " selected"
            html += (htmlopt % (value, selected, label))
        html += '</select>\n'
        return html


class HTMLTextInput(object):
    def __init__(self, name, size, maxlength, value, onchange=None):
        self.name = name
        self.size = size
        self.maxlength = maxlength
        self.value = value
        self.onchange = onchange
        self.html = '<input name="%s" type="text" size="%d" \
maxlength="%d" value="%s" %s>\n'

    def getHTML(self):
        onchg = cond(self.onchange is None,
                     ('', 'onchange="%s"' % self.onchange))
        return self.html % (self.name, self.size,
                            self.maxlength, self.value, onchg)


class HTMLTable(object):
    def __init__(self, rows, border=1, headings=True,
                 cellspacing=0, cellpadding=4):
        self.rows = rows
        self.border = int(border)
        self.cellpadding = int(cellpadding)
        self.cellspacing = int(cellspacing)
        self.headings = headings

    def getHTML(self):
        if self.rows == []:
            return ""
        html = '<table border="%d" cellpadding="%d" cellspacing="%d">\n' \
               % (self.border, self.cellpadding, self.cellspacing)
        rowcount = 0
        for row in self.rows:
            html += '<tr>\n'
            tag = 'td'
            if rowcount == 0 and self.headings:
                tag = 'th'
            for cell in row:
                span = 1
                if isinstance(cell, tuple):
                    span = cell[1]
                    cell = cell[0]
                html += '<%s colspan="%d">' % (tag, span)
                if hasattr(cell, 'getHTML'):
                    html += cell.getHTML()
                else:
                    html += str(cell)
                html += '</%s>\n' % (tag, )
            html += '</tr>\n'
            rowcount += 1
        html += '</table>\n'
        return html


class HTMLJSButton(object):
    def __init__(self, tag, action):
        self.tag = tag
        self.action = action
        self.html = """<div onclick="%s" """\
                    """onmouseover="document.body.style.cursor='pointer';" """\
                    """onmouseout="document.body.style.cursor='default';"  """\
                    """><u>%s</u></div>"""

    def getHTML(self):
        return self.html % (self.action, self.tag)


class HTMLImage(object):
    def __init__(self, src, width, height, alt):
        self.src = src
        self.width = width
        self.height = height
        self.alt = alt
        self.html = '<img src="%s" alt="%s" width="%d" height="%d" />'

    def getHTML(self):
        return self.html % (self.src, self.alt, self.width, self.height)


class HTMLStatus(object):
    def __init__(self, status):
        self.status = status

    def getHTML(self):
        return HTMLImage('/images/%s.gif' % self.status,
                         width=12, height=12, alt=self.status).getHTML()


class HTMLSummaryLink(HTMLLink):
    def __init__(self, emd5_tag):
        qry = urlencode({'id': emd5_tag[0]})
        HTMLLink.__init__(self, '/summary?' + qry, emd5_tag[1])


class HTMLFCScheme(object):
    def __init__(self, fc_id, kn):
        self.dom = kn.km.search(['shortname', 'latex_unit'],
                                {'type': 'field', 'dim_of': {'id': fc_id}})
        self.rng = kn.km.search(['shortname', 'latex_unit'],
                                {'type': 'field', 'id': fc_id})[0]
        self.latex = '$%s$(%s)[%s]'
        self.html = """<pre class="LaTeX">%s</pre>"""

    def getSpaced(self):
        domstr = ''
        for shortname, unit in self.dom:
            domstr += "$%s$[%s],&nbsp;&nbsp;" % (shortname, unit)
        return self.latex  % (self.rng[0], domstr[:-13], self.rng[1])

    def getHTML(self):
        return self.html % self.getSpaced()


class HTMLSCScheme(object):
    def __init__(self, sc_id, kn):
        columns = kn.km.search(['id'], {'type': 'field',
                                        'col_of': {'id': sc_id}})
        self.fc_schemes = [HTMLFCScheme(col[0], kn) for col in columns]
        self.shortname = kn.km.search(['shortname'], {'type': 'sample',
                                                      'id': sc_id})[0][0]
        self.html = """<pre class="LaTeX">%s</pre>"""

    def getSpaced(self):
        lstr = '$%s$(' % self.shortname
        for fc_scheme in self.fc_schemes:
            lstr += fc_scheme.getSpaced() + ',&nbsp;&nbsp;'
        return lstr[:-13] + ')'

    def getHTML(self):
        return self.html % self.getSpaced()


class HTMLChildrenTable(HTMLTable):
    def __init__(self, dc_id, kn):
        child = cond(dc_id.endswith('field'), ('dim_of', 'col_of'))
        result = kn.km.search(
            ['id', 'longname'], {'type': 'field', child: {'id': dc_id}})
        rows = [[HTMLSummaryLink(res) for res in result]]
        HTMLTable.__init__(self, rows, headings=False)


class HTMLAttrTable(HTMLTable):
    def __init__(self, dc_id, kn):
        with SQLiteWrapper(kn.km.dbase) as wrapper:
            attrs = wrapper[dc_id]['attributes']
        rows = [('attribute', 'value')]
        rows.extend(attrs.items())
        HTMLTable.__init__(self, rows)


class HTMLBrowseBar(HTMLTable):
    def __init__(self, offset, limit):
        rows = [[HTMLJSButton('--previous--',
                              'set_offset(%d);' % (offset - limit, )),
                 'offset: %d' % offset,
                 HTMLJSButton('--next--',
                              'set_offset(%d);' % (offset + limit, ))]]
        HTMLTable.__init__(self, rows, border=0, headings=False)


class WebInterface(object):
    """
    Web interface for the KnowledgeNode class.
    """

    anystr = '-- any --'

    def __init__(self, knowledge_node, enabled):
        """
        knowledge_node -- KN instance the web interface is bound to
        enabled -- whether the web interface should be enabled upon start.
        """
        self.enabled = enabled
        self.kn = knowledge_node
        self.rootdir = pkg_resources.resource_filename('pyphant', 'web')
        self.url_link = HTMLLink(self.kn.url, self.kn.url).getHTML()
        self.menu = HTMLTable(
            [[HTMLLink('/search?shorten=True', 'Browse Data Containers'),
              HTMLLink('/remotes/', 'Manage Remotes'),
              HTMLLink('/log/', 'Show Log')
              ]], headings=False, border=0).getHTML()
        self._setup_routes()

    def _setup_routes(self):
        self.kn.app.add_route('/', self.frontpage)
        self.kn.app.add_route('/images/:filename', self.images)
        self.kn.app.add_route('/remote_action', self.remote_action)
        self.kn.app.add_route('/favicon.ico', self.favicon)
        self.kn.app.add_route('/log/', self.log)
        self.kn.app.add_route('/remotes/', self.remotes)
        self.kn.app.add_route('/summary', self.summary)
        self.kn.app.add_route('/script/:filename', self.script)
        self.kn.app.add_route('/search', self.search)

    def frontpage(self):
        if not self.enabled:
            return template('disabled')
        return template('frontpage',
                        local_url=self.url_link,
                        local_uuid=self.kn.uuid[9:],
                        menu=self.menu)

    def remotes(self):
        if not self.enabled:
            return template('disabled')
        remote_rows = [[('URL', 2), 'UUID', ('Action', 2)]]
        for remote in self.kn.remotes:
            endisstr = cond(remote._status == 2, ('enable', 'disable'))
            qdict = {
                'host': remote.host, 'port': remote.port, 'action': endisstr
                }
            endis = HTMLLink('/remote_action?' + urlencode(qdict), endisstr)
            qdict['action'] = 'remove'
            rem = HTMLLink('/remote_action?' + urlencode(qdict), 'remove')
            uuid = remote.uuid
            if uuid != None:
                uuid = uuid[9:]
            remote_rows.append([HTMLStatus(remote.status),
                                HTMLLink(remote.url, remote.url),
                                uuid, endis, rem])
        remote_table = HTMLTable(remote_rows)
        return template('remotes', remote_table=remote_table.getHTML())

    def images(self, filename):
        if not self.enabled:
            return template('disabled')
        send_file(filename, os.path.join(self.rootdir, 'images'),
                  guessmime=False,
                  mimetype=self.kn.mimetypes.guess_type(filename)[0])

    def script(self, filename):
        if not self.enabled:
            return template('disabled')
        send_file(filename, os.path.join(self.rootdir, 'script'),
                  guessmime=False,
                  mimetype='application/javascript')

    def remote_action(self):
        if not self.enabled:
            return template('disabled')
        qry = request.GET
        action_dict = {'enable': self.kn.enable_remote,
                       'disable': self.kn.disable_remote,
                       'remove': self.kn.remove_remote,
                       'add': self.kn.register_remote}
        valk = validate_keys(['host', 'port', 'action'], qry)
        if not valk is True:
            return valk
        try:
            validate([(qry['port'],
                       lambda p: p.isdigit() and int(p) < 65536,
                       "Parameter 'port' has to be between 0 and 65535."),
                      (qry['action'],
                       lambda a: a in action_dict,
                       "Invalid value for parameter 'action': %s")]
                     )
        except ValueError, valerr:
            return template('message', heading='Parameter Error',
                            message=valerr.args[0], back_url='/remotes/')
        port = int(qry['port'])
        try:
            action_dict[qry['action']](qry['host'], port)
        except RemoteError, remerr:
            return template('message', heading='Error', message=remerr.args[0],
                            back_url='/remotes/')
        return self.remotes()

    def favicon(self):
        return self.images('favicon.ico')

    def log(self):
        if not self.enabled:
            return template('disabled')
        with open(os.path.join(getPyphantPath(), 'pyphant.log')) as logfile:
            loglines = logfile.readlines()
        return template('log', loglines=''.join(loglines), url=self.url_link)

    def summary(self):
        if not self.enabled:
            return template('disabled')
        qry = request.GET
        val_id = validate_keys(['id'], qry)
        if not val_id is True:
            return val_id
        if qry['id'].endswith('.field'):
            return self.fieldcontainer(qry['id'])
        elif qry['id'].endswith('.sample'):
            return self.samplecontainer(qry['id'])
        else:
            return template(
                'message', heading='Parameter Error', back_url='/',
                message="Not an emd5: '%s'" % qry['id'])

    def common_summary(self, dc_id):
        dctype = cond(dc_id.endswith('field'), ('field', 'sample'))
        keys = ['id', 'machine', 'creator', 'date', 'hash', 'longname']
        result = self.kn.km.search(keys, {'type': dctype, 'id': dc_id})
        if result == []:
            raise DCNotFoundError(template(
                'message', heading='Parameter Error', back_url='/',
                message="Could not find data container '%s'" % dc_id))
        return zip(keys, result[0])

    def fieldcontainer(self, fc_id):
        try:
            common_rows = self.common_summary(fc_id)
        except DCNotFoundError, dcnferr:
            return dcnferr.args[0]
        rows = [['scheme', HTMLFCScheme(fc_id, self.kn)]]
        rows.extend(common_rows[:-1])
        rows.append(['dimensions', HTMLChildrenTable(fc_id, self.kn)])
        rows.append(['attributes', HTMLAttrTable(fc_id, self.kn)])
        htmlsumm = HTMLTable(rows, headings=False)
        return template('fieldcontainer', summary=htmlsumm,
                        longname=common_rows[5][1])

    def samplecontainer(self, sc_id):
        try:
            common_rows = self.common_summary(sc_id)
        except DCNotFoundError, dcnferr:
            return dcnferr.args[0]
        scheme = HTMLSCScheme(sc_id, self.kn)
        rows = [['scheme', scheme]]
        rows.extend(common_rows[:-1])
        rows.append(['columns', HTMLChildrenTable(sc_id, self.kn)])
        rows.append(['attributes', HTMLAttrTable(sc_id, self.kn)])
        htmlsumm = HTMLTable(rows, headings=False)
        return template('samplecontainer', summary=htmlsumm,
                        longname=common_rows[5][1])

    def search(self):
        if not self.enabled:
            return template('disabled')
        # --- qry verification and completion ---
        common_keys = ['type', 'machine', 'creator', 'longname', 'shortname']
        complete = dict([(key, self.anystr) for key in common_keys])
        complete.update(
            {'order_by': 'date', 'order_asc': 'True', 'offset': '0',
             'jump': 'False', 'date_from': '', 'date_to': '',
             'shorten': 'False', 'add_attr': 'False',
             'rem_attr': 'None'}
            )
        qry = request.GET
        for key in complete:
            if not key in qry:
                qry[key] = complete[key]
        do_shorten = (qry['shorten'] == 'True')
        add_attr = (qry['add_attr'] == 'True')
        order_asc = (qry['order_asc'] == 'True')
        order_by = qry['order_by']
        body_onload = cond(
            qry['jump'] == 'True',
            (""" onload="window.location.hash='result_view';" """, ""))
        offset = max(int(qry['offset']), 0)
        limit = 500
        search_dict = dict([(key, qry[key]) for key in common_keys \
                            if qry[key] != self.anystr])
        if qry['date_from'] != '':
            search_dict['date_from'] = qry['date_from']
        if qry['date_to'] != '':
            search_dict['date_to'] = qry['date_to']
        attr_post = [key[8:] for key in qry if key.startswith('attr_key')]
        attr_post.sort(key=lambda x: int(x))
        if qry['rem_attr'] != 'None':
            attr_post.remove(qry['rem_attr'])
            qry.pop('attr_key' + qry['rem_attr'])
            qry.pop('attr_value' + qry['rem_attr'])
        if add_attr:
            if attr_post == []:
                last_index = -1
            else:
                last_index = int(attr_post[-1])
            attr_post.append(str(last_index + 1))
            qry['attr_key' + attr_post[-1]] = ''
            qry['attr_value' + attr_post[-1]] = ''
        from pyphant.core.Helpers import utf82uc

        def testany(str1):
            str2 = utf82uc(str1)
            if str2 == u"":
                str2 = self.kn.km.any_value
            return str2

        search_dict['attributes'] = dict(
            [(qry['attr_key' + apost],
              testany(qry['attr_value' + apost])) \
                 for apost in attr_post]
            )
        #print search_dict
        # --- common search keys ---
        optionss = [[(self.anystr, )] \
                    + self.kn.km.search([key], search_dict, distinct=True) \
                    for key in common_keys]
        rows = [common_keys[:3]]
        rows.append([HTMLDropdown(
            key, [opt[0] for opt in opts],
            qry[key], "document.search_form.submit();") \
                     for key, opts in zip(common_keys[:3], optionss[:3])])
        common = HTMLTable(rows).getHTML()
        rows = [common_keys[3:]]
        rows.append([HTMLDropdown(
            key, [opt[0] for opt in opts],
            qry[key], "document.search_form.submit();") \
                     for key, opts in zip(common_keys[3:], optionss[3:])])
        common += "<br />" + HTMLTable(rows).getHTML()
        # --- date search keys ---
        date_table = HTMLTable(
            [['date from', 'date to'],
             [HTMLTextInput('date_from', 26, 26, qry['date_from'],
                            "document.search_form.submit();"),
              HTMLTextInput('date_to', 26, 26, qry['date_to'],
                            "document.search_form.submit();")]])
        date = date_table.getHTML()
        # --- attribute search keys
        rows = [['attribute', 'value',
                 HTMLJSButton('--add--', 'add_attribute();')]]
        rows.extend([[HTMLTextInput('attr_key' + apost,
                                    20, 100, qry['attr_key' + apost]),
                      HTMLTextInput('attr_value' + apost,
                                    50, 1000, qry['attr_value' + apost],
                                    'document.search_form.submit();'),
                      HTMLJSButton('--remove--',
                                   "remove_attribute('%s');" % (apost, ))] \
                     for apost in attr_post])
        attributes = HTMLTable(rows).getHTML()
        # --- results ---
        missing_keys = ['date'] \
                       + [key for key in common_keys \
                          if qry[key] == self.anystr] \
                       + ['id']
        if not order_by in missing_keys:
            order_by = 'date'
        search_result = self.kn.km.search(
            missing_keys, search_dict, order_by=order_by,
            order_asc=order_asc, limit=limit, offset=offset)
        rows = [order_bar(missing_keys[:-1], order_by, order_asc)\
                + ['details', 'tmp']]
        rows.extend([nice(srow[:-1], missing_keys[:-1], do_shorten) \
                     + (HTMLSummaryLink((srow[-1], 'click')),
                        self.kn.km.isTemporary(srow[-1])) \
                     for srow in search_result])
        bbar = HTMLBrowseBar(offset, limit).getHTML()
        result = bbar + "<br />" + HTMLTable(rows).getHTML() + "<br />" + bbar
        return template('search',
                        common=common,
                        date=date,
                        attributes=attributes,
                        special='special...',
                        result=result,
                        order_by=order_by,
                        order_asc=qry['order_asc'],
                        body_onload=body_onload,
                        shorten=cond(do_shorten, ('checked', '')),
                        offset=qry['offset'])



# -*- coding: utf-8 -*-

# Copyright (c) 2008-2009, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""


enc=lambda s: unicode(s, "utf-8")

import platform,os,socket,datetime
from pyphant.core import FMFGen
import numpy
import pyphant.core.Helpers

def dtype2colFormat(dtype):
    if dtype.name.startswith('float'):
        return "%e"
    elif dtype.name.startswith('int'):
        return "%i"
    else:
        print dtype
        return "%s"

def field2fmf(fieldContainer):
    factory = FMFGen.gen_factory(out_coding='utf-8', eol='\n')
    fc = factory.gen_fmf()
    fc.add_reference_item('author', pyphant.core.Helpers.getUsername())
    fc.add_reference_item('title',fieldContainer.longname)
    fc.add_reference_item('place',socket.getfqdn())
    fc.add_reference_item('created',datetime.datetime.utcnow().isoformat())
    sec = factory.gen_section("parameters")
    for key,value in fieldContainer.attributes.iteritems():
        if type(value)==type([]):
            output = ' '.join(value)
        else:
            output = str(value)
        sec.add_item(key,output)
    fc.add_section(sec)
    if len(fieldContainer.data.shape)==1:
        dim = fieldContainer.dimensions[0]
        if fieldContainer.error == None:
            data = numpy.vstack([dim.data, fieldContainer.data])
        else:
            data = numpy.vstack([dim.data, fieldContainer.data,fieldContainer.error])
        tab = factory.gen_table(data.transpose())
        tab.add_column_def(dim.longname, dim.shortname, str(dim.unit))
        if fieldContainer.error == None:
            errorSymbol = None
        else:
            errorSymbol = u"\\Delta_{%s}" % fieldContainer.shortname
        tab.add_column_def(fieldContainer.longname,
                           fieldContainer.shortname,
                           str(fieldContainer.unit),
                           dependencies = [dim.shortname],
                           error = errorSymbol)
        if fieldContainer.error != None:
            tab.add_column_def(u"uncertainty of %s" % fieldContainer.longname,
                               errorSymbol,
                               str(fieldContainer.unit))
    elif fieldContainer.dimensions[0].isIndex():
        dim = fieldContainer.dimensions[-1]
        try:
            data = numpy.vstack([dim.data, fieldContainer.data])
        except:
            print dim.data,fieldContainer
        tab = factory.gen_table(data.transpose())
        tab.add_column_def(dim.longname, dim.shortname, str(dim.unit), format=dtype2colFormat(dim.data.dtype))
        superscript = ('st','nd','rd')
        for i in xrange(len(fieldContainer.dimensions[0].data)):
            if i<3:
                sup = superscript[i]
            else:
                sup = 'th'
            longname = "%i$^%s$ %s" % (i+1,sup,fieldContainer.longname)
            shortname = "%s_%i" % (fieldContainer.shortname,i+1)
            tab.add_column_def(longname,shortname,
                               str(fieldContainer.unit),
                               dependencies = [dim.shortname],
                               format=dtype2colFormat(fieldContainer.data.dtype))
    fc.add_table(tab)
    return str(fc)



# -*- coding: utf-8 -*-

# Copyright (c) 2008, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""Provides unittests for ImageVisualizers. Because it is difficult to check the correct visualization of a image, it is just checked, wether the visualizers run without assertions."""

from pyphant.mplbackend import ensure_mpl_backend
ensure_mpl_backend('agg')

import sys, os.path
import tempfile
import unittest


import numpy
from pyphant.visualizers.Chart import LineChart,ScatterPlot
from pyphant.core import DataContainer as DC
from pyphant.tests import Helpers

outputFormat = 'ps'

class TestLinePlot(unittest.TestCase):
    """Create a one-dimensional field and check the problem-free generation of a line plot."""
    def setUp(self):
        self.m = 10
        self.n = 100
        self.kappa1=0.0
        self.tmpdir = tempfile.gettempdir()
        self.visualizer = LineChart

    def testVisualization(self):
        X = numpy.linspace(-1.5,1.5,self.n)
        self.lambDim = 1.0
        xField = DC.FieldContainer(X,
                                   unit = '1 m',
                                   longname = 'position',
                                   shortname = 'x')
        self.V = DC.FieldContainer(-self.lambDim/2* X**2 + X**4/4-X*self.kappa1,
                                   unit='1 V',dimensions=[xField],
                                   longname = 'electric potential',
                                   shortname=r'\varphi',
                                   attributes={'title':'testVisualization'})
        self.V.seal()
        visualizer = self.visualizer(self.V,show=False)
        filename = os.path.join(self.tmpdir,'pyphant-'+DC.parseId(self.V.id)[0]+'%s.%s' % (visualizer.name,outputFormat))
        visualizer.figure.savefig(filename.replace(' ',''))

    def testErrorVisualization(self):
        X = numpy.linspace(-1.5,1.5,self.n)
        self.lambDim = 1.0
        xField = DC.FieldContainer(X,
                                   unit = '1 m',
                                   longname = 'position',
                                   shortname = 'x')
        self.V = DC.FieldContainer(-self.lambDim/2* X**2 + X**4/4-X*self.kappa1,
                                   unit='1 V',dimensions=[xField],
                                   longname = 'electric potential',
                                   shortname=r'\varphi',
                                   attributes={'title':'testErrorVisualization'})
        self.V.error = 0.1*numpy.abs(self.V.data)
        self.V.seal()
        visualizer = self.visualizer(self.V,show=False)
        filename = os.path.join(self.tmpdir,'pyphant-'+DC.parseId(self.V.id)[0]+'%s.%s' % (visualizer.name,outputFormat))
        visualizer.figure.savefig(filename.replace(' ',''))

    def testIntersectionXArray(self):
        X,LAMB = numpy.meshgrid(numpy.linspace(-1.5,1.5,self.n),
                                numpy.linspace(-1.0,1.0,self.m))
        self.lambDim = numpy.linspace(-1.0,1.0,self.m)
        self.xDim = numpy.linspace(-1.5,1.5,self.n)
        lambField = DC.FieldContainer(self.lambDim,
                                      unit = '1 V / m**3',
                                      longname='parameter',
                                      shortname='\lambda')
        xField = DC.FieldContainer(self.xDim,
                                   unit = '1 m',
                                   longname = 'position',
                                   shortname = 'x')
        #Prepare potential
        V = []
        for i in xrange(len(lambField.data)):
            u = X[i]
            V.append(-lambField.data[i]/2* u**2 + u**4/4-u*self.kappa1)
        self.V = DC.FieldContainer(numpy.array(V),unit='1 V',dimensions=[lambField,xField],
                                   longname = 'electric potential',
                                   shortname=r'\varphi')

        self.V.seal()
        visualizer = self.visualizer(self.V,show=False)
        filename = os.path.join(self.tmpdir,'pyphant-'+DC.parseId(self.V.id)[0]+'%s.%s' % (visualizer.name,outputFormat))
        visualizer.figure.savefig(filename.replace(' ',''))


    def testIntersectionXVector(self):
        X,LAMB = numpy.meshgrid(numpy.linspace(-1.5,1.5,self.n),
                                numpy.linspace(-1.0,1.0,self.m))
        self.lambDim = LAMB[:,0]
        self.xDim = numpy.linspace(-1.5,1.5,self.n)
        lambField = DC.FieldContainer(self.lambDim,
                                      unit = '1 V / m**3',
                                      longname='parameter',
                                      shortname=r'\lambda')
        xField = DC.FieldContainer(self.xDim,
                                   unit = '1 m',
                                   longname = 'position',
                                   shortname = 'x')
        #Prepare potential
        V = []
        for i in xrange(len(lambField.data)):
            u = X[i]
            V.append(-lambField.data[i]/2* u**2 + u**4/4-u*self.kappa1)
        self.V = DC.FieldContainer(numpy.array(V),unit='1 V',dimensions=[lambField, xField],
                                   longname = 'electric potential',
                                   shortname=r'\varphi',
                                   attributes={'title':'testIntersectionXVector'})
        self.V.seal()
        visualizer = self.visualizer(self.V,show=False)
        filename = os.path.join(self.tmpdir,'pyphant-'+DC.parseId(self.V.id)[0]+'%s.%s' % (visualizer.name,outputFormat))
        visualizer.figure.savefig(filename.replace(' ',''))

    def testTableIncludingNan(self):
        X,LAMB = numpy.meshgrid(numpy.linspace(-1.5,1.5,self.n),
                                numpy.linspace(-1.0,1.0,self.m))
        self.lambDim = LAMB[:,0]
        self.xDim = numpy.linspace(-1.5,1.5,self.n)
        lambField = DC.FieldContainer(self.lambDim,
                                      unit = '1 V / m**3',
                                      longname='parameter',
                                      shortname=r'\lambda')
        xField = DC.FieldContainer(self.xDim,
                                   unit = '1 m',
                                   longname = 'position',
                                   shortname = 'x')
        x0,curv,mask = Helpers.fixedPoints(lambField.data,kappa1=self.kappa1)
        fixedPoints = DC.FieldContainer(numpy.array(x0).transpose(),
                                        unit = xField.unit,
                                        dimensions=[DC.generateIndex(0,3), lambField],
                                        longname = 'position of the local extrema of electric potential',
                                        shortname = 'x_0',
                                        attributes={'title':'testTableIncludingNan'})
        fixedPoints.seal()
        visualizer = self.visualizer(fixedPoints,show=False)
        filename = os.path.join(self.tmpdir,'pyphant-'+DC.parseId(fixedPoints.id)[0]+'%s.%s' % (visualizer.name,outputFormat))
        visualizer.figure.savefig(filename.replace(' ',''))

    def testTableIncludingNanAndErrors(self):
        X,LAMB = numpy.meshgrid(numpy.linspace(-1.5,1.5,self.n),
                                numpy.linspace(-1.0,1.0,self.m))
        self.lambDim = LAMB[:,0]
        self.xDim = numpy.linspace(-1.5,1.5,self.n)
        lambField = DC.FieldContainer(self.lambDim,
                                      unit = '1 V / m**3',
                                      longname='parameter',
                                      shortname=r'\lambda')
        xField = DC.FieldContainer(self.xDim,
                                   unit = '1 m',
                                   longname = 'position',
                                   shortname = 'x')
        x0,curv,mask = Helpers.fixedPoints(lambField.data,kappa1=self.kappa1)
        fixedPoints = DC.FieldContainer(numpy.array(x0).transpose(),
                                        unit = xField.unit,
                                        dimensions=[DC.generateIndex(0,3), lambField],
                                        longname = 'position of the local extrema of electric potential',
                                        shortname = 'x_0',
                                        attributes={'title':'testTableIncludingNanAndErrors'})
        fixedPoints.error = 0.1 * fixedPoints.data
        fixedPoints.seal()
        visualizer = self.visualizer(fixedPoints,show=False)
        filename = os.path.join(self.tmpdir,'pyphant-'+DC.parseId(fixedPoints.id)[0]+'%s.%s' % (visualizer.name,outputFormat))
        visualizer.figure.savefig(filename.replace(' ',''))

class TestScatterPlot(TestLinePlot):
    """Create a one-dimensional field and check the problem-free generation of a scatter plot."""
    def setUp(self):
        self.m = 10
        self.n = 100
        self.kappa1=0.0
        self.tmpdir = tempfile.gettempdir()
        self.visualizer = ScatterPlot

if __name__ == '__main__':
    unittest.main()
